from ._differential_equation_form import DifferentialEquationForm
from ._first_order_ode_form import FirstOrderODEForm
from ._second_order_ode_form import SecondOrderODEForm
from ._heat_equation_form import HeatEquationForm
from ._wave_equation_form import WaveEquationForm
