app_name = "Differential Equation Solver"
solution_recorded_message = "Your solution has been recorded in {0}"

# Boundary types
dirichlet = "Dirichlet"
neumann = "Neumann"
left_boundary_type = "Left Boundary Type"
right_boundary_type = "Right Boundary Type"

# Button text
solve = "Solve"
export = "Export"
show_plot = "Show Plot"
play = "Play"
pause = "Pause"
